==> 0.3.78.tar.gz: lib/Widget/filter_widget/filter_widget.dart <==
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Controller/filter_controller.dart';
import 'package:wowit/Controller/load_more_base_controller.dart';
import 'package:wowit/Controller/searched_controller.dart';
import 'package:wowit/Widget/baseWidgets.dart';
import 'package:wowit/Widget/filter_widget/util.dart';

import '../../message.dart';
import 'component/filter_screen.dart';
import 'component/seached_list.dart';
import 'component/search_screen.dart';

typedef OnMapSortationValueChanged = void Function(
  String fieldName,
  ESortMode eSortMode,
);
typedef OnValueChanged = void Function(
  String fieldName,
  Object value,
);

typedef SortationValueGetter = ESortMode Function(String fieldName);
typedef ConditionValueGetter = Object Function(String fieldName);

typedef DataLoader = Future<Map<String, List<Map<String, Object>>>> Function();

typedef GetConditionView = Widget Function(
  Map<String, Object> mapCondition,
  OnValueChanged onValueChanged,
);

typedef GetLineView = Widget Function(Map<String, Object> dataItem);

abstract class FilterWidget<T extends BaseController> extends BaseWidget<T> {
  final bool isFilterOnServer;
  final String title;
  final Color color;
  final FilterController filterController;
  final String fieldNameSearching;
  final EdgeInsets searchedListPadding;
  final Widget filterIcon;
  final SearchedDataLoader loadSearchedData;
  final bool isComponent;

  FilterWidget(
    BaseController baseController, {
    this.fieldNameSearching,
    this.title,
    this.color,
    this.searchedListPadding,
    this.filterIcon,
    this.isFilterOnServer = false,
    this.isComponent = true,
    @required List<FilterItem> filterItems,
  })  : filterController = FilterController(filterItems),
        loadSearchedData = (baseController is SearchedController)
            ? (baseController as SearchedController)?.loadSearchedData
            : null,
        super(baseController as T,isComponent:isComponent);

  void loadMore() {
    final baseController = getController<T>();
    if (baseController is LoadMoreBaseController) {
      baseController.loadMore();
    } else {
      print('Please extends LoadMoreController');
    }
  }

  Widget getConditionView(
    Map<String, Object> mapCondition,
    void onValueChange(String fieldName, Object value),
  );

  Widget getLineView(Map<String, Object> dataItem) {
    print('FilterWidget: Need to provide a LineView');
    return Text('Need to provide a LineView (Override getLineView())');
  }

  void _onApply() {
    if (isFilterOnServer) {
      applyFilterOnServer();
    } else {
      applyFilterBaseOnList();
    }
    Get.back();
  }

  void onSortationValueChanged(String fieldName, ESortMode sortMode) {
    filterController.updateMapSortation(fieldName, sortMode);
  }

  void onConditionValueChanged(String fieldName, Object value) {
    filterController.updateMapCondition(fieldName, value);
  }

  void goToFilterView() {
    Get.to(
      () => FilterScreen(
        getConditionView: getConditionView,
        filterController: filterController,
        title: this.title,
        onApply: _onApply,
        onClear: _onClear,
        color: this.color,
      ),
    );
  }

  void goToSearchView() {
    print('loadSearchedData: $loadSearchedData');
    Get.to(
      () => SearchScreen(
        searchedListModel: SearchedListModel(
          getConditionView: getConditionView,
          filterItems: filterController.filterItems,
          getLineView: getLineView,
          title: title,
          searchedListPadding: searchedListPadding,
          isFilterOnServer: this.isFilterOnServer,
        ),
        loadSearchedData: loadSearchedData,
        fieldNameSearching: fieldNameSearching,
        filterIcon: filterIcon,
      ),
    );
  }

  // Lọc dựa trên data đã load về.
  Future<void> applyFilterBaseOnList() async {
    print('${applyFilterBaseOnList.runtimeType} applySort');
    final baseController = getController<T>();

    final loadedData = await baseController.loadData();

    final filterDataFieldName = getFilterFieldName(loadedData);

    final filteredData =
        filterController.applyFilter(loadedData[filterDataFieldName]);
    loadedData[filterDataFieldName] = filteredData;
    baseController.setData(loadedData);
    baseController.update();
  }

  // Sắp xếp dựa trên data đã load về.
  void applySortBaseOnList() {
    final baseController = getController<T>();
    print('${baseController.runtimeType} applySort');

    final loadedData = baseController.getData();

    final filterDataFieldName = getFilterFieldName(loadedData);

    final filteredData =
        filterController.applySort(loadedData[filterDataFieldName]);

    loadedData[filterDataFieldName] = filteredData;
    baseController.setData(loadedData);
    baseController.update();
  }

  // Sắp xếp dựa trên procedure trên server
  void applySortOnServer() {
    final mapSortation = filterController.mapSortation;
    final fieldName = mapSortation.keys.first;
    final sortMode = mapSortation[fieldName];

    final controller = getController<T>();

    if (controller is LoadMoreBaseController) {
      controller.sort(fieldName, sortMode == ESortMode.smallestToLargest);
    } else {
      print('Please extends LoadMoreController');
    }
  }

  // Lọc dựa trên procedure trên server
  void applyFilterOnServer() {
    final controller = getController<T>();
    if (controller is LoadMoreBaseController) {
      controller.filter(filterController.filterMapCondition);
    } else {
      print('Please extends LoadMoreController');
    }
  }

  Widget getLoadingView() {
    return Padding(
      padding: const EdgeInsets.only(top: 10, bottom: 40),
      child: Center(
        child: CircularProgressIndicator(),
      ),
    );
  }

  Widget getEndOfPageView() {
    return Container(
      padding: const EdgeInsets.only(top: 24, bottom: 24),
      decoration: BoxDecoration(
        color: Color(0xff2a367e),
        borderRadius: BorderRadius.all(Radius.circular(10)),
      ),
      child: Center(
        child: Text(
          Message.getMessage('FilterWidget_ban_da_den_cuoi_trang'),
          style: TextStyle(
            color: Colors.white,
            fontWeight: FontWeight.bold,
          ),
        ),
      ),
    );
  }

  void _onClear() {
    filterController.clear();
  }

  Object getValueConditionByFieldName(String fieldName) {
    return filterController.mapCondition[fieldName];
  }

  ESortMode getValueSortationByFieldName(String fieldName) {
    return filterController.mapSortation[fieldName];
  }
}

class FilterItem {
  final String fieldName;

  final EFilterMode filterMode;

  ///Gia tri mac dinh
  ///Nen cung cap gia tri mac dinh khi [filterMode] == [EFilterMode.range]
  final Object defaultValue;

  //Gia tri khi chon 'tat ca'
  //Gap gia tri nay thi se in ra toan bo list ma khong filter.
  final Object allValue;

  //Gia tri max khi EFilterMode = inRangeOrGreaterThanMax
  final Object maxValue;

  const FilterItem({
    @required this.fieldName,
    @required this.filterMode,
    this.maxValue,
    this.defaultValue,
    this.allValue,
  });
}

enum EFilterMode {
  equal,
  range,
  contain,
  lessThan,
  lessOrEqual,
  greaterThan,
  greaterOrEqual,
  inRangeOrGreaterThanMax,
  insensitiveAndUnsignedContain
}

enum ESortMode {
  smallestToLargest,
  largestToSmallest,
}

==> 0.3.85.tar.gz: lib/Widget/filter_widget/filter_widget.dart <==
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Controller/filter_controller.dart';
import 'package:wowit/Controller/load_more_base_controller.dart';
import 'package:wowit/Controller/searched_controller.dart';
import 'package:wowit/Widget/baseWidgets.dart';
import 'package:wowit/Widget/filter_widget/util.dart';

import '../../message.dart';
import 'component/filter_screen.dart';
import 'component/seached_list.dart';
import 'component/search_screen.dart';

typedef OnMapSortationValueChanged = void Function(
  String fieldName,
  ESortMode eSortMode,
);
typedef OnValueChanged = void Function(
  String fieldName,
  Object value,
);

typedef SortationValueGetter = ESortMode Function(String fieldName);
typedef ConditionValueGetter = Object Function(String fieldName);

typedef DataLoader = Future<Map<String, List<Map<String, Object>>>> Function();

typedef GetConditionView = Widget Function(
  Map<String, Object> mapCondition,
  OnValueChanged onValueChanged,
);

typedef GetLineView = Widget Function(Map<String, Object> dataItem);

abstract class FilterWidget<T extends BaseController> extends BaseWidget<T> {
  final bool isFilterOnServer;
  final String title;
  final Color color;
  final FilterController filterController;
  final String fieldNameSearching;
  final EdgeInsets searchedListPadding;
  final Widget filterIcon;
  final SearchedDataLoader loadSearchedData;
  final bool isComponent;
  final String tag;

  FilterWidget(
    BaseController baseController, {
    this.fieldNameSearching,
    this.title,
    this.color,
    this.searchedListPadding,
    this.filterIcon,
    this.isFilterOnServer = false,
    this.isComponent = true,
    this.tag,
    @required List<FilterItem> filterItems,
  })  : filterController = FilterController(filterItems),
        loadSearchedData = (baseController is SearchedController)
            ? (baseController as SearchedController)?.loadSearchedData
            : null,
        super(baseController as T,isComponent:isComponent,tag: tag);

  void loadMore() {
    final baseController = getController<T>(tag: tag);
    if (baseController is LoadMoreBaseController) {
      baseController.loadMore();
    } else {
      print('Please extends LoadMoreController');
    }
  }

  Widget getConditionView(
    Map<String, Object> mapCondition,
    void onValueChange(String fieldName, Object value),
  );

  Widget getLineView(Map<String, Object> dataItem) {
    print('FilterWidget: Need to provide a LineView');
    return Text('Need to provide a LineView (Override getLineView())');
  }

  void _onApply() {
    if (isFilterOnServer) {
      applyFilterOnServer();
    } else {
      applyFilterBaseOnList();
    }
    Get.back();
  }

  void onSortationValueChanged(String fieldName, ESortMode sortMode) {
    filterController.updateMapSortation(fieldName, sortMode);
  }

  void onConditionValueChanged(String fieldName, Object value) {
    filterController.updateMapCondition(fieldName, value);
  }

  void goToFilterView() {
    Get.to(
      () => FilterScreen(
        getConditionView: getConditionView,
        filterController: filterController,
        title: this.title,
        onApply: _onApply,
        onClear: _onClear,
        color: this.color,
      ),
    );
  }

  void goToSearchView() {
    print('loadSearchedData: $loadSearchedData');
    Get.to(
      () => SearchScreen(
        searchedListModel: SearchedListModel(
          getConditionView: getConditionView,
          filterItems: filterController.filterItems,
          getLineView: getLineView,
          title: title,
          searchedListPadding: searchedListPadding,
          isFilterOnServer: this.isFilterOnServer,
        ),
        loadSearchedData: loadSearchedData,
        fieldNameSearching: fieldNameSearching,
        filterIcon: filterIcon,
      ),
    );
  }

  // Lọc dựa trên data đã load về.
  Future<void> applyFilterBaseOnList() async {
    print('${applyFilterBaseOnList.runtimeType} applySort');
    final baseController = getController<T>(tag: tag);

    final loadedData = await baseController.loadData();

    final filterDataFieldName = getFilterFieldName(loadedData);

    final filteredData =
        filterController.applyFilter(loadedData[filterDataFieldName]);
    loadedData[filterDataFieldName] = filteredData;
    baseController.setData(loadedData);
    baseController.update();
  }

  // Sắp xếp dựa trên data đã load về.
  void applySortBaseOnList() {
    final baseController = getController<T>(tag: tag);
    print('${baseController.runtimeType} applySort');

    final loadedData = baseController.getData();

    final filterDataFieldName = getFilterFieldName(loadedData);

    final filteredData =
        filterController.applySort(loadedData[filterDataFieldName]);

    loadedData[filterDataFieldName] = filteredData;
    baseController.setData(loadedData);
    baseController.update();
  }

  // Sắp xếp dựa trên procedure trên server
  void applySortOnServer() {
    final mapSortation = filterController.mapSortation;
    final fieldName = mapSortation.keys.first;
    final sortMode = mapSortation[fieldName];

    final controller = getController<T>(tag: tag);

    if (controller is LoadMoreBaseController) {
      controller.sort(fieldName, sortMode == ESortMode.smallestToLargest);
    } else {
      print('Please extends LoadMoreController');
    }
  }

  // Lọc dựa trên procedure trên server
  void applyFilterOnServer() {
    final controller = getController<T>(tag: tag);
    if (controller is LoadMoreBaseController) {
      controller.filter(filterController.filterMapCondition);
    } else {
      print('Please extends LoadMoreController');
    }
  }

  Widget getLoadingView() {
    return Padding(
      padding: const EdgeInsets.only(top: 10, bottom: 40),
      child: Center(
        child: CircularProgressIndicator(),
      ),
    );
  }

  Widget getEndOfPageView() {
    return Container(
      padding: const EdgeInsets.only(top: 24, bottom: 24),
      decoration: BoxDecoration(
        color: Color(0xff2a367e),
        borderRadius: BorderRadius.all(Radius.circular(10)),
      ),
      child: Center(
        child: Text(
          Message.getMessage('FilterWidget_ban_da_den_cuoi_trang'),
          style: TextStyle(
            color: Colors.white,
            fontWeight: FontWeight.bold,
          ),
        ),
      ),
    );
  }

  void _onClear() {
    filterController.clear();
  }

  Object getValueConditionByFieldName(String fieldName) {
    return filterController.mapCondition[fieldName];
  }

  ESortMode getValueSortationByFieldName(String fieldName) {
    return filterController.mapSortation[fieldName];
  }
}

class FilterItem {
  final String fieldName;

  final EFilterMode filterMode;

  ///Gia tri mac dinh
  ///Nen cung cap gia tri mac dinh khi [filterMode] == [EFilterMode.range]
  final Object defaultValue;

  //Gia tri khi chon 'tat ca'
  //Gap gia tri nay thi se in ra toan bo list ma khong filter.
  final Object allValue;

  //Gia tri max khi EFilterMode = inRangeOrGreaterThanMax
  final Object maxValue;

  const FilterItem({
    @required this.fieldName,
    @required this.filterMode,
    this.maxValue,
    this.defaultValue,
    this.allValue,
  });
}

enum EFilterMode {
  equal,
  range,
  contain,
  lessThan,
  lessOrEqual,
  greaterThan,
  greaterOrEqual,
  inRangeOrGreaterThanMax,
  insensitiveAndUnsignedContain
}

enum ESortMode {
  smallestToLargest,
  largestToSmallest,
}
